from __future__ import unicode_literals, print_function, absolute_import, division

import hashlib

from django.db import models
from django.core.files.storage import default_storage
from django.core.files import File
from django.conf import settings


class UploadManager(models.Manager):

    def upload_file(self, uploaded_file, category=''):
        sha256 = hashlib.sha256()
        for c in uploaded_file.chunks():
            sha256.update(c)
        checksum = sha256.hexdigest()
        inst = self.get_queryset().filter(checksum=checksum).first()

        if inst:
            return inst

        inst = self.model(
            checksum=checksum,
            category=category,
            name=uploaded_file.name,
            size=uploaded_file.size,
            content_type=uploaded_file.content_type,
        )

        filename = '{}.{}'.format(checksum, uploaded_file.content_type.split('/')[1])
        inst.upload.save(filename, File(uploaded_file))

        return inst


class AbstractUpload(models.Model):
    checksum = models.CharField(max_length=64, primary_key=True)
    category = models.CharField(max_length=255, db_index=True, default='')
    name = models.CharField(max_length=255)
    content_type = models.CharField(max_length=255, default='applicaiton/binary')
    size = models.BigIntegerField(default=0)
    created_on = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.name

    def get_url(self):
        return self.upload.url

    def to_dict(self):
        return {
            'checksum': self.checksum,
            'url': self.get_url(),
            'content_type': self.content_type,
            'size': self.size,
            'name': self.name,
            'created_on': self.created_on.isoformat(),
        }


def _upload_to(instance, filename):
    if instance.category:
        filename = '{}/{}'.format(instance.category.strip().replace(' ', '-'), filename)
    return '{}/{}'.format(getattr(settings, 'UPMAN_UPLOAD_PATH', 'uploads/upman'), filename)


class Upload(AbstractUpload):
    upload = models.FileField(
        upload_to=_upload_to,
        storage=default_storage
    )

    objects = UploadManager()


class ImageUpload(AbstractUpload):
    upload = models.ImageField(
        upload_to=_upload_to,
        storage=default_storage,
        width_field='width',
        height_field='height',
    )
    height = models.PositiveIntegerField(default=0)
    width = models.PositiveIntegerField(default=0)

    objects = UploadManager()




from __future__ import unicode_literals, print_function, absolute_import, division

import json

from django.http import HttpResponse

from .models import Upload, ImageUpload
from .forms import UploadForm, ImageUploadForm


def _uploads(request, Model, Form):
    category = request.GET.get('category', '').strip().lower()

    if request.method == 'POST':
        form = Form(request.POST, request.FILES)
        if form.is_valid():
            upload = Model.objects.upload_file(form.cleaned_data['file'], category=category)
            return HttpResponse(json.dumps(upload.to_dict()), content_type='application/json')
        else:
            errors = dict((key, unicode(val[0])) for key, val in form.errors.items())
            return HttpResponse(json.dumps(errors), content_type='application/json', status=400)

    results = []
    uploads = Model.objects.filter(category=category)

    if request.GET.get('search'):
        uploads = uploads.filter(name__icontains=request.GET['search'].strip())

    for upload in uploads:
        results.append(upload.to_dict())

    return HttpResponse(json.dumps(results), content_type='application/json')


def uploads(request):
    return _uploads(request, Upload, UploadForm)


def image_uploads(request):
    return _uploads(request, ImageUpload, ImageUploadForm)




from __future__ import unicode_literals, print_function, absolute_import, division

from django import forms


class UploadForm(forms.Form):
    file = forms.FileField()


class ImageUploadForm(forms.Form):
    file = forms.ImageField()




from __future__ import print_function, division, absolute_import, unicode_literals

from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^uploads$', views.uploads, name='upman-uploads'),
    url(r'^image-uploads$', views.image_uploads, name='upman-image-uploads'),
]



import codecs
from setuptools import setup


with codecs.open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

version = '0.1.0'

setup(
    name='django-upman',
    packages=['django_upman'],
    version=version,
    description='Django uploads manager',
    long_description=readme,
    author='Igor Partola',
    author_email='igor@igorpartola.com',
    license='MIT License',
    keywords='Django uploads manager',
    install_requires=['django', 'Pillow'],
    package_data={'django_upman': [
        '../README.md',
        '../LICENSE',
    ]},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ]
)
